/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
// TODO: Include PageTemplateModels here too?? Probably
 */


/**
 * @class
 * Initializes a new instance of the SharedAccessSignatureAuthorizationRule class.
 * @constructor
 * The properties that describe the keys to access the IotHub artifacts.
 * @member {string} [keyName] The name of the key.
 * 
 * @member {string} [primaryKey] The primary key.
 * 
 * @member {string} [secondaryKey] The secondary key.
 * 
 * @member {string} [rights] The access rights. Possible values include:
 * 'RegistryRead', 'RegistryWrite', 'ServiceConnect', 'DeviceConnect',
 * 'RegistryRead, RegistryWrite', 'RegistryRead, ServiceConnect',
 * 'RegistryRead, DeviceConnect', 'RegistryWrite, ServiceConnect',
 * 'RegistryWrite, DeviceConnect', 'ServiceConnect, DeviceConnect',
 * 'RegistryRead, RegistryWrite, ServiceConnect', 'RegistryRead,
 * RegistryWrite, DeviceConnect', 'RegistryRead, ServiceConnect,
 * DeviceConnect', 'RegistryWrite, ServiceConnect, DeviceConnect',
 * 'RegistryRead, RegistryWrite, ServiceConnect, DeviceConnect'
 * 
 */
export interface SharedAccessSignatureAuthorizationRule {
    keyName?: string;
    primaryKey?: string;
    secondaryKey?: string;
    rights?: string;
}

/**
 * @class
 * Initializes a new instance of the IotHubProperties class.
 * @constructor
 * The Iot Hub properties.
 * @member {array} [authorizationPolicies] The authorization rules.
 * 
 * @member {string} [hostName] The name of the host.
 * 
 * @member {object} [eventHubEndpoints] The event hub endpoint properties.
 * 
 * @member {object} [storageEndpoints] The list of storage end points where
 * files can be uploaded. Currently only one storage account can be
 * configured.
 * 
 * @member {object} [messagingEndpoints] The list of messaging end points
 * configured.
 * 
 * @member {boolean} [enableFileUploadNotifications] The flag which indicates
 * whether file upload notification should be enabled. This is optional at
 * iot hub level. When enabled upload notifications will be available.
 * 
 * @member {object} [cloudToDevice]
 * 
 * @member {number} [cloudToDevice.maxDeliveryCount] The max delivery count
 * for the device queue. Range : 1-100.
 * 
 * @member {moment.duration} [cloudToDevice.defaultTtlAsIso8601] The default
 * time to live for the device queue. Range : 1 Min (PT1M) - 2 Days (P2D).
 * 
 * @member {object} [cloudToDevice.feedback]
 * 
 * @member {moment.duration} [cloudToDevice.feedback.lockDurationAsIso8601]
 * The lock duration for the feedback queue. Range: 5 Sec (PT5S) - 5 Min
 * (PT5M).
 * 
 * @member {moment.duration} [cloudToDevice.feedback.ttlAsIso8601] The time to
 * live for the feedback queue. Range: 1 Min (PT1M) - 2 Days (P2D).
 * 
 * @member {number} [cloudToDevice.feedback.maxDeliveryCount] The max delivery
 * count. Range : 1-100.
 * 
 * @member {string} [comments] The comments.
 * 
 * @member {object} [operationsMonitoringProperties]
 * 
 * @member {object} [operationsMonitoringProperties.events]
 * 
 * @member {string} [features] The Capabilities/Features that need to be
 * enabled for the Hub. Possible values include: 'None', 'DeviceManagement'
 * 
 */
export interface IotHubProperties {
    authorizationPolicies?: SharedAccessSignatureAuthorizationRule[];
    hostName?: string;
    eventHubEndpoints?: { [propertyName: string]: EventHubProperties };
    storageEndpoints?: { [propertyName: string]: StorageEndpointProperties };
    messagingEndpoints?: { [propertyName: string]: MessagingEndpointProperties };
    enableFileUploadNotifications?: boolean;
    cloudToDevice?: CloudToDeviceProperties;
    comments?: string;
    operationsMonitoringProperties?: OperationsMonitoringProperties;
    features?: string;
}

/**
 * @class
 * Initializes a new instance of the EventHubProperties class.
 * @constructor
 * The properties of the provisioned event hub used by the Iot Hub.
 * @member {number} [retentionTimeInDays] The retention time in days. Range of
 * values [For F1: 1-1, S1: 1-7, S2: 1-7, S3: 1-7].
 * 
 * @member {number} [partitionCount] The partition count. Range of values [For
 * F1: 2-2, S1: 2-128, S2: 2-128, S3: 2-128].
 * 
 * @member {array} [partitionIds] The partition ids.
 * 
 * @member {string} [path] The eventhub path.
 * 
 * @member {string} [endpoint] The endpoint.
 * 
 * @member {array} [internalAuthorizationPolicies] The internal authorization
 * rules.
 * 
 * @member {array} [authorizationPolicies] The authorization rules.
 * 
 */
export interface EventHubProperties {
    retentionTimeInDays?: number;
    partitionCount?: number;
    partitionIds?: string[];
    path?: string;
    endpoint?: string;
    internalAuthorizationPolicies?: SharedAccessAuthorizationRule[];
    authorizationPolicies?: SharedAccessAuthorizationRule[];
}

/**
 * @class
 * Initializes a new instance of the SharedAccessAuthorizationRule class.
 * @constructor
 * @member {string} [keyName] The key name.
 * 
 * @member {string} [primaryKey] The primary key.
 * 
 * @member {string} [issuerName] The issuer name.
 * 
 * @member {string} [secondaryKey] The secondary key.
 * 
 * @member {string} [claimType] The claim type.
 * 
 * @member {string} [claimValue] The claim value.
 * 
 * @member {array} [rights] The rights.
 * 
 * @member {date} [createdTime] The created time.
 * 
 * @member {date} [modifiedTime] The modified time.
 * 
 * @member {number} [revision] The revision.
 * 
 */
export interface SharedAccessAuthorizationRule {
    keyName?: string;
    primaryKey?: string;
    issuerName?: string;
    secondaryKey?: string;
    claimType?: string;
    claimValue?: string;
    rights?: string[];
    createdTime?: Date;
    modifiedTime?: Date;
    revision?: number;
}

/**
 * @class
 * Initializes a new instance of the StorageEndpointProperties class.
 * @constructor
 * The properties of the Storage Endpoint for file upload.
 * @member {moment.duration} [sasTtlAsIso8601] SAS time to live. Range: 1 Min
 * (PT1M) - 1 Day (P1D).
 * 
 * @member {string} [connectionString] The account key credentials for storage
 * account selected by customer for uploading files.
 * 
 * @member {string} [containerName] The root container name where all files
 * will be uploaded.
 * 
 */
export interface StorageEndpointProperties {
    sasTtlAsIso8601?: moment.Duration;
    connectionString?: string;
    containerName?: string;
}

/**
 * @class
 * Initializes a new instance of the MessagingEndpointProperties class.
 * @constructor
 * The properties of the Messaging Endpoints used by this IoT Hub.
 * @member {moment.duration} [lockDurationAsIso8601] The lock duration. Range:
 * 5 Sec (PT5S) - 5 Min (PT5M).
 * 
 * @member {moment.duration} [ttlAsIso8601] The time to live. Range: 1 Min
 * (PT1M) - 2 Days (P2D).
 * 
 * @member {number} [maxDeliveryCount] The max delivery count. Range: 1-100.
 * 
 */
export interface MessagingEndpointProperties {
    lockDurationAsIso8601?: moment.Duration;
    ttlAsIso8601?: moment.Duration;
    maxDeliveryCount?: number;
}

/**
 * @class
 * Initializes a new instance of the CloudToDeviceProperties class.
 * @constructor
 * The Iot Hub Cloud-To-Device messaging properties.
 * @member {number} [maxDeliveryCount] The max delivery count for the device
 * queue. Range : 1-100.
 * 
 * @member {moment.duration} [defaultTtlAsIso8601] The default time to live
 * for the device queue. Range : 1 Min (PT1M) - 2 Days (P2D).
 * 
 * @member {object} [feedback]
 * 
 * @member {moment.duration} [feedback.lockDurationAsIso8601] The lock
 * duration for the feedback queue. Range: 5 Sec (PT5S) - 5 Min (PT5M).
 * 
 * @member {moment.duration} [feedback.ttlAsIso8601] The time to live for the
 * feedback queue. Range: 1 Min (PT1M) - 2 Days (P2D).
 * 
 * @member {number} [feedback.maxDeliveryCount] The max delivery count. Range
 * : 1-100.
 * 
 */
export interface CloudToDeviceProperties {
    maxDeliveryCount?: number;
    defaultTtlAsIso8601?: moment.Duration;
    feedback?: FeedbackProperties;
}

/**
 * @class
 * Initializes a new instance of the FeedbackProperties class.
 * @constructor
 * The Feedback Queue properties.
 * @member {moment.duration} [lockDurationAsIso8601] The lock duration for the
 * feedback queue. Range: 5 Sec (PT5S) - 5 Min (PT5M).
 * 
 * @member {moment.duration} [ttlAsIso8601] The time to live for the feedback
 * queue. Range: 1 Min (PT1M) - 2 Days (P2D).
 * 
 * @member {number} [maxDeliveryCount] The max delivery count. Range : 1-100.
 * 
 */
export interface FeedbackProperties {
    lockDurationAsIso8601?: moment.Duration;
    ttlAsIso8601?: moment.Duration;
    maxDeliveryCount?: number;
}

/**
 * @class
 * Initializes a new instance of the OperationsMonitoringProperties class.
 * @constructor
 * The Operation Monitoring properties.
 * @member {object} [events]
 * 
 */
export interface OperationsMonitoringProperties {
    events?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the IotHubSkuInfo class.
 * @constructor
 * The Sku related information for the hub.
 * @member {string} [name] The name of the Sku. Possible values include: 'F1',
 * 'S1', 'S2', 'S3'
 * 
 * @member {string} [tier] The tier. Possible values include: 'Free',
 * 'Standard'
 * 
 * @member {number} [capacity] The number of units being provisioned. Range of
 * values [For F1: 1-1, S1: 1-200, S2: 1-200, S3: 1-10]. To go above this
 * range, call support.
 * 
 */
export interface IotHubSkuInfo {
    name?: string;
    tier?: string;
    capacity?: number;
}

/**
 * @class
 * Initializes a new instance of the Resource class.
 * @constructor
 * @member {string} [id] The Resource Id.
 * 
 * @member {string} [name] The Resource name.
 * 
 * @member {string} [type] The Resource type.
 * 
 * @member {string} [location] The Resource location.
 * 
 * @member {object} [tags] The Resource tags.
 * 
 */
export interface Resource extends BaseResource {
    id?: string;
    name?: string;
    type?: string;
    location?: string;
    tags?: { [propertyName: string]: string };
}

/**
 * @class
 * Initializes a new instance of the IotHubDescription class.
 * @constructor
 * The description of the IotHub.
 * @member {string} [subscriptionid] The subscription identifier.
 * 
 * @member {string} [resourcegroup] The resource group name uniquely
 * identifies the resource group within the user subscriptionId.
 * 
 * @member {string} [etag] The Etag field is *not* required. If it is provided
 * in the response body, it must also be provided as a header per the normal
 * ETag convention.
 * 
 * @member {object} [properties]
 * 
 * @member {array} [properties.authorizationPolicies] The authorization rules.
 * 
 * @member {string} [properties.hostName] The name of the host.
 * 
 * @member {object} [properties.eventHubEndpoints] The event hub endpoint
 * properties.
 * 
 * @member {object} [properties.storageEndpoints] The list of storage end
 * points where files can be uploaded. Currently only one storage account can
 * be configured.
 * 
 * @member {object} [properties.messagingEndpoints] The list of messaging end
 * points configured.
 * 
 * @member {boolean} [properties.enableFileUploadNotifications] The flag which
 * indicates whether file upload notification should be enabled. This is
 * optional at iot hub level. When enabled upload notifications will be
 * available.
 * 
 * @member {object} [properties.cloudToDevice]
 * 
 * @member {number} [properties.cloudToDevice.maxDeliveryCount] The max
 * delivery count for the device queue. Range : 1-100.
 * 
 * @member {moment.duration} [properties.cloudToDevice.defaultTtlAsIso8601]
 * The default time to live for the device queue. Range : 1 Min (PT1M) - 2
 * Days (P2D).
 * 
 * @member {object} [properties.cloudToDevice.feedback]
 * 
 * @member {moment.duration}
 * [properties.cloudToDevice.feedback.lockDurationAsIso8601] The lock
 * duration for the feedback queue. Range: 5 Sec (PT5S) - 5 Min (PT5M).
 * 
 * @member {moment.duration} [properties.cloudToDevice.feedback.ttlAsIso8601]
 * The time to live for the feedback queue. Range: 1 Min (PT1M) - 2 Days
 * (P2D).
 * 
 * @member {number} [properties.cloudToDevice.feedback.maxDeliveryCount] The
 * max delivery count. Range : 1-100.
 * 
 * @member {string} [properties.comments] The comments.
 * 
 * @member {object} [properties.operationsMonitoringProperties]
 * 
 * @member {object} [properties.operationsMonitoringProperties.events]
 * 
 * @member {string} [properties.features] The Capabilities/Features that need
 * to be enabled for the Hub. Possible values include: 'None',
 * 'DeviceManagement'
 * 
 * @member {object} [sku]
 * 
 * @member {string} [sku.name] The name of the Sku. Possible values include:
 * 'F1', 'S1', 'S2', 'S3'
 * 
 * @member {string} [sku.tier] The tier. Possible values include: 'Free',
 * 'Standard'
 * 
 * @member {number} [sku.capacity] The number of units being provisioned.
 * Range of values [For F1: 1-1, S1: 1-200, S2: 1-200, S3: 1-10]. To go above
 * this range, call support.
 * 
 */
export interface IotHubDescription extends Resource {
    subscriptionid?: string;
    resourcegroup?: string;
    etag?: string;
    properties?: IotHubProperties;
    sku?: IotHubSkuInfo;
}

/**
 * @class
 * Initializes a new instance of the ErrorDetails class.
 * @constructor
 * The properties related to the details of an error.
 * @member {string} [code] The error code.
 * 
 * @member {string} [httpStatusCode] The http status code.
 * 
 * @member {string} [message] The error message.
 * 
 * @member {string} [details] The error details.
 * 
 */
export interface ErrorDetails {
    code?: string;
    httpStatusCode?: string;
    message?: string;
    details?: string;
}

/**
 * @class
 * Initializes a new instance of the IotHubQuotaMetricInfo class.
 * @constructor
 * The properties related to quota metrics.
 * @member {string} [name] The name of the quota metric.
 * 
 * @member {number} [currentValue] The current value for the quota metric.
 * 
 * @member {number} [maxValue] The maximum value of the quota metric.
 * 
 */
export interface IotHubQuotaMetricInfo {
    name?: string;
    currentValue?: number;
    maxValue?: number;
}

/**
 * @class
 * Initializes a new instance of the RegistryStatistics class.
 * @constructor
 * The properties related to the registry statistics.
 * @member {number} [totalDeviceCount] The total device count.
 * 
 * @member {number} [enabledDeviceCount] The enabled device count.
 * 
 * @member {number} [disabledDeviceCount] The disabled device count.
 * 
 */
export interface RegistryStatistics {
    totalDeviceCount?: number;
    enabledDeviceCount?: number;
    disabledDeviceCount?: number;
}

/**
 * @class
 * Initializes a new instance of the JobResponse class.
 * @constructor
 * The properties describing a Job Response.
 * @member {string} [jobId] The job identifier.
 * 
 * @member {date} [startTimeUtc] Start time of the Job.
 * 
 * @member {date} [endTimeUtc] Represents the time the job stopped processing.
 * 
 * @member {string} [type] The type of job to execute. Possible values
 * include: 'unknown', 'export', 'import', 'backup', 'readDeviceProperties',
 * 'writeDeviceProperties', 'updateDeviceConfiguration', 'rebootDevice',
 * 'factoryResetDevice', 'firmwareUpdate'
 * 
 * @member {string} [status] Status of the Job. Possible values include:
 * 'unknown', 'enqueued', 'running', 'completed', 'failed', 'cancelled'
 * 
 * @member {string} [failureReason] If status == failure, this represents a
 * string containing the reason.
 * 
 * @member {string} [statusMessage] The status message for the job.
 * 
 * @member {string} [deviceId] The deviceId related to this response.
 * 
 * @member {string} [parentJobId] The jobId of the parent job, if any.
 * 
 */
export interface JobResponse {
    jobId?: string;
    startTimeUtc?: Date;
    endTimeUtc?: Date;
    type?: string;
    status?: string;
    failureReason?: string;
    statusMessage?: string;
    deviceId?: string;
    parentJobId?: string;
}

/**
 * @class
 * Initializes a new instance of the IotHubSkuDescription class.
 * @constructor
 * The properties related to the Sku.
 * @member {string} [resourceType] The type of the resource.
 * 
 * @member {object} [sku]
 * 
 * @member {string} [sku.name] The name of the Sku. Possible values include:
 * 'F1', 'S1', 'S2', 'S3'
 * 
 * @member {string} [sku.tier] The tier. Possible values include: 'Free',
 * 'Standard'
 * 
 * @member {number} [sku.capacity] The number of units being provisioned.
 * Range of values [For F1: 1-1, S1: 1-200, S2: 1-200, S3: 1-10]. To go above
 * this range, call support.
 * 
 * @member {object} [capacity]
 * 
 * @member {number} [capacity.minimum] The minimum number of units.
 * 
 * @member {number} [capacity.maximum] The maximum number of units.
 * 
 * @member {number} [capacity.default] The default number of units.
 * 
 * @member {string} [capacity.scaleType] The type of the scale. Possible
 * values include: 'Automatic', 'Manual', 'None'
 * 
 */
export interface IotHubSkuDescription {
    resourceType?: string;
    sku?: IotHubSkuInfo;
    capacity?: IotHubCapacity;
}

/**
 * @class
 * Initializes a new instance of the IotHubCapacity class.
 * @constructor
 * The properties related to the capacity information.
 * @member {number} [minimum] The minimum number of units.
 * 
 * @member {number} [maximum] The maximum number of units.
 * 
 * @member {number} [default] The default number of units.
 * 
 * @member {string} [scaleType] The type of the scale. Possible values
 * include: 'Automatic', 'Manual', 'None'
 * 
 */
export interface IotHubCapacity {
    minimum?: number;
    maximum?: number;
    default?: number;
    scaleType?: string;
}

/**
 * @class
 * Initializes a new instance of the EventHubConsumerGroupInfo class.
 * @constructor
 * The properties indicating the EventHubConsumerGroupInfo.
 * @member {object} [tags] The tags.
 * 
 * @member {string} [id] The event hub consumer group identifier.
 * 
 * @member {string} [name] The event hub consumer group name.
 * 
 */
export interface EventHubConsumerGroupInfo {
    tags?: { [propertyName: string]: string };
    id?: string;
    name?: string;
}

/**
 * @class
 * Initializes a new instance of the OperationInputs class.
 * @constructor
 * OperationInputs.
 * @member {string} [name] The name of the iot hub.
 * 
 */
export interface OperationInputs {
    name?: string;
}

/**
 * @class
 * Initializes a new instance of the IotHubNameAvailabilityInfo class.
 * @constructor
 * The properties indicating whether a given iothub name is available.
 * @member {boolean} [nameAvailable] The value which indicates whether the
 * provided name is available.
 * 
 * @member {string} [reason] The reason for unavailability. Possible values
 * include: 'Invalid', 'AlreadyExists'
 * 
 * @member {string} [message] The message describing the detailed reason.
 * 
 */
export interface IotHubNameAvailabilityInfo {
    nameAvailable?: boolean;
    reason?: string;
    message?: string;
}

/**
 * @class
 * Initializes a new instance of the ExportDevicesRequest class.
 * @constructor
 * Used to provide parameters when requesting an export of all devices in the
 * hub.
 * @member {string} exportBlobContainerUri The export BLOB container URI.
 * 
 * @member {boolean} excludeKeys The value indicating whether keys should be
 * excluded during export.
 * 
 */
export interface ExportDevicesRequest {
    exportBlobContainerUri: string;
    excludeKeys: boolean;
}

/**
 * @class
 * Initializes a new instance of the ImportDevicesRequest class.
 * @constructor
 * Used to provide parameters when requesting an import of all devices in the
 * hub.
 * @member {string} inputBlobContainerUri The input BLOB container URI.
 * 
 * @member {string} outputBlobContainerUri The output BLOB container URI.
 * 
 */
export interface ImportDevicesRequest {
    inputBlobContainerUri: string;
    outputBlobContainerUri: string;
}
