/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

var util = require('util');

/**
 * @class
 * Initializes a new instance of the IotHubDescription class.
 * @constructor
 * The description of the IotHub.
 * @member {string} [subscriptionid] The subscription identifier.
 * 
 * @member {string} [resourcegroup] The resource group name uniquely
 * identifies the resource group within the user subscriptionId.
 * 
 * @member {string} [etag] The Etag field is *not* required. If it is provided
 * in the response body, it must also be provided as a header per the normal
 * ETag convention.
 * 
 * @member {object} [properties]
 * 
 * @member {array} [properties.authorizationPolicies] The authorization rules.
 * 
 * @member {string} [properties.hostName] The name of the host.
 * 
 * @member {object} [properties.eventHubEndpoints] The event hub endpoint
 * properties.
 * 
 * @member {object} [properties.storageEndpoints] The list of storage end
 * points where files can be uploaded. Currently only one storage account can
 * be configured.
 * 
 * @member {object} [properties.messagingEndpoints] The list of messaging end
 * points configured.
 * 
 * @member {boolean} [properties.enableFileUploadNotifications] The flag which
 * indicates whether file upload notification should be enabled. This is
 * optional at iot hub level. When enabled upload notifications will be
 * available.
 * 
 * @member {object} [properties.cloudToDevice]
 * 
 * @member {number} [properties.cloudToDevice.maxDeliveryCount] The max
 * delivery count for the device queue. Range : 1-100.
 * 
 * @member {moment.duration} [properties.cloudToDevice.defaultTtlAsIso8601]
 * The default time to live for the device queue. Range : 1 Min (PT1M) - 2
 * Days (P2D).
 * 
 * @member {object} [properties.cloudToDevice.feedback]
 * 
 * @member {moment.duration}
 * [properties.cloudToDevice.feedback.lockDurationAsIso8601] The lock
 * duration for the feedback queue. Range: 5 Sec (PT5S) - 5 Min (PT5M).
 * 
 * @member {moment.duration} [properties.cloudToDevice.feedback.ttlAsIso8601]
 * The time to live for the feedback queue. Range: 1 Min (PT1M) - 2 Days
 * (P2D).
 * 
 * @member {number} [properties.cloudToDevice.feedback.maxDeliveryCount] The
 * max delivery count. Range : 1-100.
 * 
 * @member {string} [properties.comments] The comments.
 * 
 * @member {object} [properties.operationsMonitoringProperties]
 * 
 * @member {object} [properties.operationsMonitoringProperties.events]
 * 
 * @member {string} [properties.features] The Capabilities/Features that need
 * to be enabled for the Hub. Possible values include: 'None',
 * 'DeviceManagement'
 * 
 * @member {object} [sku]
 * 
 * @member {string} [sku.name] The name of the Sku. Possible values include:
 * 'F1', 'S1', 'S2', 'S3'
 * 
 * @member {string} [sku.tier] The tier. Possible values include: 'Free',
 * 'Standard'
 * 
 * @member {number} [sku.capacity] The number of units being provisioned.
 * Range of values [For F1: 1-1, S1: 1-200, S2: 1-200, S3: 1-10]. To go above
 * this range, call support.
 * 
 */
function IotHubDescription() {
  IotHubDescription['super_'].call(this);
}

util.inherits(IotHubDescription, models['Resource']);

/**
 * Defines the metadata of IotHubDescription
 *
 * @returns {object} metadata of IotHubDescription
 *
 */
IotHubDescription.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'IotHubDescription',
    type: {
      name: 'Composite',
      className: 'IotHubDescription',
      modelProperties: {
        id: {
          required: false,
          readOnly: true,
          serializedName: 'id',
          type: {
            name: 'String'
          }
        },
        name: {
          required: false,
          readOnly: true,
          serializedName: 'name',
          constraints: {
            Pattern: '^(?![0-9]+$)(?!-)[a-zA-Z0-9-]{2,49}[a-zA-Z0-9]$'
          },
          type: {
            name: 'String'
          }
        },
        type: {
          required: false,
          readOnly: true,
          serializedName: 'type',
          type: {
            name: 'String'
          }
        },
        location: {
          required: false,
          serializedName: 'location',
          type: {
            name: 'String'
          }
        },
        tags: {
          required: false,
          serializedName: 'tags',
          type: {
            name: 'Dictionary',
            value: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        },
        subscriptionid: {
          required: false,
          serializedName: 'subscriptionid',
          type: {
            name: 'String'
          }
        },
        resourcegroup: {
          required: false,
          serializedName: 'resourcegroup',
          type: {
            name: 'String'
          }
        },
        etag: {
          required: false,
          serializedName: 'etag',
          type: {
            name: 'String'
          }
        },
        properties: {
          required: false,
          serializedName: 'properties',
          type: {
            name: 'Composite',
            className: 'IotHubProperties'
          }
        },
        sku: {
          required: false,
          serializedName: 'sku',
          type: {
            name: 'Composite',
            className: 'IotHubSkuInfo'
          }
        }
      }
    }
  };
};

module.exports = IotHubDescription;
