/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator 0.15.0.0
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var util = require('util');

/**
 * @class
 * Initializes a new instance of the SiteAuthSettings class.
 * @constructor
 * Configuration settings for the Azure App Service Authentication /
 * Authorization feature.
 * @member {boolean} [enabled] Gets or sets a value indicating whether the
 * Authentication / Authorization feature is enabled for the current app.
 * 
 * @member {string} [httpApiPrefixPath] Gets or sets the relative path prefix
 * used by platform HTTP APIs.
 * Changing this value is not recommended except for compatibility
 * reasons.
 * 
 * @member {string} [unauthenticatedClientAction] Gets or sets the action to
 * take when an unauthenticated client attempts to access the app. Possible
 * values include: 'RedirectToLoginPage', 'AllowAnonymous'
 * 
 * @member {boolean} [tokenStoreEnabled] Gets or sets a value indicating
 * whether to durably store platform-specific security tokens
 * obtained during login flows. This capability is disabled by
 * default.
 * 
 * @member {array} [allowedExternalRedirectUrls] Gets or sets a collection of
 * external URLs that can be redirected to as part of logging in
 * or logging out of the web app. Note that the query string part
 * of the URL is ignored.
 * This is an advanced setting typically only needed by Windows
 * Store application backends.
 * Note that URLs within the current domain are always implicitly
 * allowed.
 * 
 * @member {string} [defaultProvider] Gets or sets the default authentication
 * provider to use when multiple providers are configured.
 * This setting is only needed if multiple providers are
 * configured and the unauthenticated client
 * action is set to "RedirectToLoginPage". Possible values
 * include: 'AzureActiveDirectory', 'Facebook', 'Google', 'MicrosoftAccount',
 * 'Twitter'
 * 
 * @member {string} [clientId] Gets or sets the Client ID of this relying
 * party application, known as the client_id.
 * This setting is required for enabling OpenID Connection
 * authentication with Azure Active Directory or
 * other 3rd party OpenID Connect providers.
 * More information on OpenID Connect:
 * http://openid.net/specs/openid-connect-core-1_0.html
 * 
 * @member {string} [clientSecret] Gets or sets the Client Secret of this
 * relying party application (in Azure Active Directory, this is also
 * referred to as the Key).
 * This setting is optional. If no client secret is configured,
 * the OpenID Connect implicit auth flow is used to authenticate end users.
 * Otherwise, the OpenID Connect Authorization Code Flow is used
 * to authenticate end users.
 * More information on OpenID Connect:
 * http://openid.net/specs/openid-connect-core-1_0.html
 * 
 * @member {string} [issuer] Gets or sets the OpenID Connect Issuer URI that
 * represents the entity which issues access tokens for this application.
 * When using Azure Active Directory, this value is the URI of the
 * directory tenant, e.g. https://sts.windows.net/{tenant-guid}/.
 * This URI is a case-sensitive identifier for the token issuer.
 * More information on OpenID Connect Discovery:
 * http://openid.net/specs/openid-connect-discovery-1_0.html
 * 
 * @member {array} [allowedAudiences] Gets or sets a list of allowed audience
 * values to consider when validating JWTs issued by
 * Azure Active Directory. Note that the
 * {Microsoft.Web.Hosting.Administration.SiteAuthSettings.ClientId} value is
 * always considered an
 * allowed audience, regardless of this setting.
 * 
 * @member {string} [aadClientId]
 * 
 * @member {string} [openIdIssuer]
 * 
 * @member {string} [googleClientId] Gets or sets the OpenID Connect Client ID
 * for the Google web application.
 * This setting is required for enabling Google Sign-In.
 * Google Sign-In documentation:
 * https://developers.google.com/identity/sign-in/web/
 * 
 * @member {string} [googleClientSecret] Gets or sets the client secret
 * associated with the Google web application.
 * This setting is required for enabling Google Sign-In.
 * Google Sign-In documentation:
 * https://developers.google.com/identity/sign-in/web/
 * 
 * @member {array} [googleOAuthScopes] Gets or sets the OAuth 2.0 scopes that
 * will be requested as part of Google Sign-In authentication.
 * This setting is optional. If not specified, "openid",
 * "profile", and "email" are used as default scopes.
 * Google Sign-In documentation:
 * https://developers.google.com/identity/sign-in/web/
 * 
 * @member {string} [facebookAppId] Gets or sets the App ID of the Facebook
 * app used for login.
 * This setting is required for enabling Facebook Login.
 * Facebook Login documentation:
 * https://developers.facebook.com/docs/facebook-login
 * 
 * @member {string} [facebookAppSecret] Gets or sets the App Secret of the
 * Facebook app used for Facebook Login.
 * This setting is required for enabling Facebook Login.
 * Facebook Login documentation:
 * https://developers.facebook.com/docs/facebook-login
 * 
 * @member {array} [facebookOAuthScopes] Gets or sets the OAuth 2.0 scopes
 * that will be requested as part of Facebook Login authentication.
 * This setting is optional.
 * Facebook Login documentation:
 * https://developers.facebook.com/docs/facebook-login
 * 
 * @member {string} [twitterConsumerKey] Gets or sets the OAuth 1.0a consumer
 * key of the Twitter application used for sign-in.
 * This setting is required for enabling Twitter Sign-In.
 * Twitter Sign-In documentation:
 * https://dev.twitter.com/web/sign-in
 * 
 * @member {string} [twitterConsumerSecret] Gets or sets the OAuth 1.0a
 * consumer secret of the Twitter application used for sign-in.
 * This setting is required for enabling Twitter Sign-In.
 * Twitter Sign-In documentation:
 * https://dev.twitter.com/web/sign-in
 * 
 * @member {string} [microsoftAccountClientId] Gets or sets the OAuth 2.0
 * client ID that was created for the app used for authentication.
 * This setting is required for enabling Microsoft Account
 * authentication.
 * Microsoft Account OAuth documentation:
 * https://dev.onedrive.com/auth/msa_oauth.htm
 * 
 * @member {string} [microsoftAccountClientSecret] Gets or sets the OAuth 2.0
 * client secret that was created for the app used for authentication.
 * This setting is required for enabling Microsoft Account
 * authentication.
 * Microsoft Account OAuth documentation:
 * https://dev.onedrive.com/auth/msa_oauth.htm
 * 
 * @member {array} [microsoftAccountOAuthScopes] Gets or sets the OAuth 2.0
 * scopes that will be requested as part of Microsoft Account authentication.
 * This setting is optional. If not specified, "wl.basic" is used
 * as the default scope.
 * Microsoft Account Scopes and permissions documentation:
 * https://msdn.microsoft.com/en-us/library/dn631845.aspx
 * 
 */
function SiteAuthSettings() {
}

/**
 * Defines the metadata of SiteAuthSettings
 *
 * @returns {object} metadata of SiteAuthSettings
 *
 */
SiteAuthSettings.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'SiteAuthSettings',
    type: {
      name: 'Composite',
      className: 'SiteAuthSettings',
      modelProperties: {
        enabled: {
          required: false,
          serializedName: 'enabled',
          type: {
            name: 'Boolean'
          }
        },
        httpApiPrefixPath: {
          required: false,
          serializedName: 'httpApiPrefixPath',
          type: {
            name: 'String'
          }
        },
        unauthenticatedClientAction: {
          required: false,
          serializedName: 'unauthenticatedClientAction',
          type: {
            name: 'Enum',
            allowedValues: [ 'RedirectToLoginPage', 'AllowAnonymous' ]
          }
        },
        tokenStoreEnabled: {
          required: false,
          serializedName: 'tokenStoreEnabled',
          type: {
            name: 'Boolean'
          }
        },
        allowedExternalRedirectUrls: {
          required: false,
          serializedName: 'allowedExternalRedirectUrls',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        },
        defaultProvider: {
          required: false,
          serializedName: 'defaultProvider',
          type: {
            name: 'Enum',
            allowedValues: [ 'AzureActiveDirectory', 'Facebook', 'Google', 'MicrosoftAccount', 'Twitter' ]
          }
        },
        clientId: {
          required: false,
          serializedName: 'clientId',
          type: {
            name: 'String'
          }
        },
        clientSecret: {
          required: false,
          serializedName: 'clientSecret',
          type: {
            name: 'String'
          }
        },
        issuer: {
          required: false,
          serializedName: 'issuer',
          type: {
            name: 'String'
          }
        },
        allowedAudiences: {
          required: false,
          serializedName: 'allowedAudiences',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        },
        aadClientId: {
          required: false,
          serializedName: 'aadClientId',
          type: {
            name: 'String'
          }
        },
        openIdIssuer: {
          required: false,
          serializedName: 'openIdIssuer',
          type: {
            name: 'String'
          }
        },
        googleClientId: {
          required: false,
          serializedName: 'googleClientId',
          type: {
            name: 'String'
          }
        },
        googleClientSecret: {
          required: false,
          serializedName: 'googleClientSecret',
          type: {
            name: 'String'
          }
        },
        googleOAuthScopes: {
          required: false,
          serializedName: 'googleOAuthScopes',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        },
        facebookAppId: {
          required: false,
          serializedName: 'facebookAppId',
          type: {
            name: 'String'
          }
        },
        facebookAppSecret: {
          required: false,
          serializedName: 'facebookAppSecret',
          type: {
            name: 'String'
          }
        },
        facebookOAuthScopes: {
          required: false,
          serializedName: 'facebookOAuthScopes',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        },
        twitterConsumerKey: {
          required: false,
          serializedName: 'twitterConsumerKey',
          type: {
            name: 'String'
          }
        },
        twitterConsumerSecret: {
          required: false,
          serializedName: 'twitterConsumerSecret',
          type: {
            name: 'String'
          }
        },
        microsoftAccountClientId: {
          required: false,
          serializedName: 'microsoftAccountClientId',
          type: {
            name: 'String'
          }
        },
        microsoftAccountClientSecret: {
          required: false,
          serializedName: 'microsoftAccountClientSecret',
          type: {
            name: 'String'
          }
        },
        microsoftAccountOAuthScopes: {
          required: false,
          serializedName: 'microsoftAccountOAuthScopes',
          type: {
            name: 'Sequence',
            element: {
                required: false,
                serializedName: 'StringElementType',
                type: {
                  name: 'String'
                }
            }
          }
        }
      }
    }
  };
};

module.exports = SiteAuthSettings;
