/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

/**
 * @class
 * Initializes a new instance of the AutoPoolSpecification class.
 * @constructor
 * @summary Specifies characteristics for a temporary 'auto pool'. The Batch
 * service will create this auto pool when the job is submitted.
 *
 * @member {string} [autoPoolIdPrefix] A prefix to be added to the unique
 * identifier when a pool is automatically created. The prefix can be up to
 * 20 characters long.
 * 
 * @member {string} poolLifetimeOption The minimum lifetime of created auto
 * pools, and how multiple jobs on a schedule are assigned to pools. Possible
 * values include: 'jobschedule', 'job', 'unmapped'
 * 
 * @member {boolean} [keepAlive] Whether to keep an auto pool alive after its
 * lifetime expires.
 * 
 * @member {object} [pool] The pool specification for the auto pool.
 * 
 * @member {string} [pool.displayName]
 * 
 * @member {string} [pool.vmSize]
 * 
 * @member {object} [pool.cloudServiceConfiguration] This property and
 * VirtualMachineConfiguration are mutually exclusive and one of the
 * properties must be specified.
 * 
 * @member {string} [pool.cloudServiceConfiguration.osFamily]
 * 
 * @member {string} [pool.cloudServiceConfiguration.targetOSVersion] The
 * default value is * which specifies the latest operating system version for
 * the specified OS family.
 * 
 * @member {string} [pool.cloudServiceConfiguration.currentOSVersion]
 * 
 * @member {object} [pool.virtualMachineConfiguration] This property and
 * CloudServiceConfiguration are mutually exclusive and one of the properties
 * must be specified.
 * 
 * @member {object} [pool.virtualMachineConfiguration.imageReference]
 * 
 * @member {string}
 * [pool.virtualMachineConfiguration.imageReference.publisher] For example,
 * Canonical or MicrosoftWindowsServer.
 * 
 * @member {string} [pool.virtualMachineConfiguration.imageReference.offer]
 * For example, UbuntuServer or WindowsServer.
 * 
 * @member {string} [pool.virtualMachineConfiguration.imageReference.sku] For
 * example, 14.04.0-LTS or 2012-R2-Datacenter.
 * 
 * @member {string} [pool.virtualMachineConfiguration.imageReference.version]
 * A value of 'latest' can be specified to select the latest version of an
 * image. If omitted, the default is 'latest'.
 * 
 * @member {string} [pool.virtualMachineConfiguration.nodeAgentSKUId] The
 * Batch node agent is a program that runs on each node in the pool, and
 * provides the command-and-control interface between the node and the Batch
 * service. There are different implementations of the node agent, known as
 * SKUs, for different operating systems.
 * 
 * @member {object} [pool.virtualMachineConfiguration.windowsConfiguration]
 * This property must not be specified if the imageReference property
 * specifies a Linux OS image.
 * 
 * @member {boolean}
 * [pool.virtualMachineConfiguration.windowsConfiguration.enableAutomaticUpdates]
 * If omitted, the default value is true.
 * 
 * @member {number} [pool.maxTasksPerNode]
 * 
 * @member {object} [pool.taskSchedulingPolicy] How tasks are distributed
 * among compute nodes in the pool.
 * 
 * @member {string} [pool.taskSchedulingPolicy.nodeFillType] Possible values
 * include: 'spread', 'pack', 'unmapped'
 * 
 * @member {moment.duration} [pool.resizeTimeout]
 * 
 * @member {number} [pool.targetDedicated]
 * 
 * @member {boolean} [pool.enableAutoScale]
 * 
 * @member {string} [pool.autoScaleFormula]
 * 
 * @member {moment.duration} [pool.autoScaleEvaluationInterval]
 * 
 * @member {boolean} [pool.enableInterNodeCommunication]
 * 
 * @member {object} [pool.networkConfiguration]
 * 
 * @member {string} [pool.networkConfiguration.subnetId] The ARM resource
 * identifier of the virtual network subnet which the compute nodes of the
 * pool will join. The virtual network must be in the same region and
 * subscription as the Azure Batch account. This property can only be
 * specified for pools created with a cloudServiceConfiguration.
 * 
 * @member {object} [pool.startTask]
 * 
 * @member {string} [pool.startTask.commandLine] The command line does not run
 * under a shell, and therefore cannot take advantage of shell features such
 * as environment variable expansion. If you want to take advantage of such
 * features, you should invoke the shell in the command line, for example
 * using "cmd /c MyCommand" in Windows or "/bin/sh -c MyCommand" in Linux.
 * 
 * @member {array} [pool.startTask.resourceFiles]
 * 
 * @member {array} [pool.startTask.environmentSettings]
 * 
 * @member {boolean} [pool.startTask.runElevated]
 * 
 * @member {number} [pool.startTask.maxTaskRetryCount]
 * 
 * @member {boolean} [pool.startTask.waitForSuccess]
 * 
 * @member {array} [pool.certificateReferences]
 * 
 * @member {array} [pool.applicationPackageReferences]
 * 
 * @member {array} [pool.metadata]
 * 
 */
function AutoPoolSpecification() {
}

/**
 * Defines the metadata of AutoPoolSpecification
 *
 * @returns {object} metadata of AutoPoolSpecification
 *
 */
AutoPoolSpecification.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'AutoPoolSpecification',
    type: {
      name: 'Composite',
      className: 'AutoPoolSpecification',
      modelProperties: {
        autoPoolIdPrefix: {
          required: false,
          serializedName: 'autoPoolIdPrefix',
          type: {
            name: 'String'
          }
        },
        poolLifetimeOption: {
          required: true,
          serializedName: 'poolLifetimeOption',
          type: {
            name: 'Enum',
            allowedValues: [ 'jobschedule', 'job', 'unmapped' ]
          }
        },
        keepAlive: {
          required: false,
          serializedName: 'keepAlive',
          type: {
            name: 'Boolean'
          }
        },
        pool: {
          required: false,
          serializedName: 'pool',
          type: {
            name: 'Composite',
            className: 'PoolSpecification'
          }
        }
      }
    }
  };
};

module.exports = AutoPoolSpecification;
