/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

/**
 * @class
 * Initializes a new instance of the PoolInformation class.
 * @constructor
 * @summary Specifies how a job should be assigned to a pool.
 *
 * @member {string} [poolId] The id of an existing pool. All the tasks of the
 * job will run on the specified pool. You must specify either poolId or
 * autoPoolSpecification, but not both.
 * 
 * @member {object} [autoPoolSpecification] Characteristics for a temporary
 * 'auto pool'. The Batch service will create this auto pool when the job is
 * submitted. You must specify either poolId or autoPoolSpecification, but
 * not both.
 * 
 * @member {string} [autoPoolSpecification.autoPoolIdPrefix] The prefix can be
 * up to 20 characters long.
 * 
 * @member {string} [autoPoolSpecification.poolLifetimeOption] Possible values
 * include: 'jobschedule', 'job', 'unmapped'
 * 
 * @member {boolean} [autoPoolSpecification.keepAlive]
 * 
 * @member {object} [autoPoolSpecification.pool]
 * 
 * @member {string} [autoPoolSpecification.pool.displayName]
 * 
 * @member {string} [autoPoolSpecification.pool.vmSize]
 * 
 * @member {object} [autoPoolSpecification.pool.cloudServiceConfiguration]
 * This property and VirtualMachineConfiguration are mutually exclusive and
 * one of the properties must be specified.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.cloudServiceConfiguration.osFamily]
 * 
 * @member {string}
 * [autoPoolSpecification.pool.cloudServiceConfiguration.targetOSVersion] The
 * default value is * which specifies the latest operating system version for
 * the specified OS family.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.cloudServiceConfiguration.currentOSVersion]
 * 
 * @member {object} [autoPoolSpecification.pool.virtualMachineConfiguration]
 * This property and CloudServiceConfiguration are mutually exclusive and one
 * of the properties must be specified.
 * 
 * @member {object}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.imageReference]
 * 
 * @member {string}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.imageReference.publisher]
 * For example, Canonical or MicrosoftWindowsServer.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.imageReference.offer]
 * For example, UbuntuServer or WindowsServer.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.imageReference.sku]
 * For example, 14.04.0-LTS or 2012-R2-Datacenter.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.imageReference.version]
 * A value of 'latest' can be specified to select the latest version of an
 * image. If omitted, the default is 'latest'.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.nodeAgentSKUId]
 * The Batch node agent is a program that runs on each node in the pool, and
 * provides the command-and-control interface between the node and the Batch
 * service. There are different implementations of the node agent, known as
 * SKUs, for different operating systems.
 * 
 * @member {object}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.windowsConfiguration]
 * This property must not be specified if the imageReference property
 * specifies a Linux OS image.
 * 
 * @member {boolean}
 * [autoPoolSpecification.pool.virtualMachineConfiguration.windowsConfiguration.enableAutomaticUpdates]
 * If omitted, the default value is true.
 * 
 * @member {number} [autoPoolSpecification.pool.maxTasksPerNode]
 * 
 * @member {object} [autoPoolSpecification.pool.taskSchedulingPolicy] How
 * tasks are distributed among compute nodes in the pool.
 * 
 * @member {string}
 * [autoPoolSpecification.pool.taskSchedulingPolicy.nodeFillType] Possible
 * values include: 'spread', 'pack', 'unmapped'
 * 
 * @member {moment.duration} [autoPoolSpecification.pool.resizeTimeout]
 * 
 * @member {number} [autoPoolSpecification.pool.targetDedicated]
 * 
 * @member {boolean} [autoPoolSpecification.pool.enableAutoScale]
 * 
 * @member {string} [autoPoolSpecification.pool.autoScaleFormula]
 * 
 * @member {moment.duration}
 * [autoPoolSpecification.pool.autoScaleEvaluationInterval]
 * 
 * @member {boolean} [autoPoolSpecification.pool.enableInterNodeCommunication]
 * 
 * @member {object} [autoPoolSpecification.pool.networkConfiguration]
 * 
 * @member {string} [autoPoolSpecification.pool.networkConfiguration.subnetId]
 * The ARM resource identifier of the virtual network subnet which the
 * compute nodes of the pool will join. The virtual network must be in the
 * same region and subscription as the Azure Batch account. This property can
 * only be specified for pools created with a cloudServiceConfiguration.
 * 
 * @member {object} [autoPoolSpecification.pool.startTask]
 * 
 * @member {string} [autoPoolSpecification.pool.startTask.commandLine] The
 * command line does not run under a shell, and therefore cannot take
 * advantage of shell features such as environment variable expansion. If you
 * want to take advantage of such features, you should invoke the shell in
 * the command line, for example using "cmd /c MyCommand" in Windows or
 * "/bin/sh -c MyCommand" in Linux.
 * 
 * @member {array} [autoPoolSpecification.pool.startTask.resourceFiles]
 * 
 * @member {array} [autoPoolSpecification.pool.startTask.environmentSettings]
 * 
 * @member {boolean} [autoPoolSpecification.pool.startTask.runElevated]
 * 
 * @member {number} [autoPoolSpecification.pool.startTask.maxTaskRetryCount]
 * 
 * @member {boolean} [autoPoolSpecification.pool.startTask.waitForSuccess]
 * 
 * @member {array} [autoPoolSpecification.pool.certificateReferences]
 * 
 * @member {array} [autoPoolSpecification.pool.applicationPackageReferences]
 * 
 * @member {array} [autoPoolSpecification.pool.metadata]
 * 
 */
function PoolInformation() {
}

/**
 * Defines the metadata of PoolInformation
 *
 * @returns {object} metadata of PoolInformation
 *
 */
PoolInformation.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'PoolInformation',
    type: {
      name: 'Composite',
      className: 'PoolInformation',
      modelProperties: {
        poolId: {
          required: false,
          serializedName: 'poolId',
          type: {
            name: 'String'
          }
        },
        autoPoolSpecification: {
          required: false,
          serializedName: 'autoPoolSpecification',
          type: {
            name: 'Composite',
            className: 'AutoPoolSpecification'
          }
        }
      }
    }
  };
};

module.exports = PoolInformation;
