/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for
 * license information.
 * 
 * Code generated by Microsoft (R) AutoRest Code Generator.
 * Changes may cause incorrect behavior and will be lost if the code is
 * regenerated.
 */

'use strict';

var models = require('./index');

/**
 * @class
 * Initializes a new instance of the TaskInformation class.
 * @constructor
 * @summary Information about a task running on a compute node.
 *
 * @member {string} [taskUrl] The URL of the task.
 * 
 * @member {string} [jobId] The id of the job to which the task belongs.
 * 
 * @member {string} [taskId] The id of the task.
 * 
 * @member {number} [subtaskId] The id of the subtask if the task is a
 * multi-instance task.
 * 
 * @member {string} taskState The current state of the task. Possible values
 * include: 'active', 'preparing', 'running', 'completed'
 * 
 * @member {object} [executionInfo] Information about the execution of the
 * task.
 * 
 * @member {date} [executionInfo.startTime] If the task has been restarted or
 * retried, this is the most recent time at which the task started running.
 * 
 * @member {date} [executionInfo.endTime] This property is set only if the
 * task is in the Completed state.
 * 
 * @member {number} [executionInfo.exitCode] This property is set only if the
 * task is in Completed state.
 * 
 * @member {object} [executionInfo.schedulingError]
 * 
 * @member {string} [executionInfo.schedulingError.category] Possible values
 * include: 'usererror', 'servererror', 'unmapped'
 * 
 * @member {string} [executionInfo.schedulingError.code]
 * 
 * @member {string} [executionInfo.schedulingError.message]
 * 
 * @member {array} [executionInfo.schedulingError.details]
 * 
 * @member {number} [executionInfo.retryCount]
 * 
 * @member {date} [executionInfo.lastRetryTime]
 * 
 * @member {number} [executionInfo.requeueCount]
 * 
 * @member {date} [executionInfo.lastRequeueTime]
 * 
 */
function TaskInformation() {
}

/**
 * Defines the metadata of TaskInformation
 *
 * @returns {object} metadata of TaskInformation
 *
 */
TaskInformation.prototype.mapper = function () {
  return {
    required: false,
    serializedName: 'TaskInformation',
    type: {
      name: 'Composite',
      className: 'TaskInformation',
      modelProperties: {
        taskUrl: {
          required: false,
          serializedName: 'taskUrl',
          type: {
            name: 'String'
          }
        },
        jobId: {
          required: false,
          serializedName: 'jobId',
          type: {
            name: 'String'
          }
        },
        taskId: {
          required: false,
          serializedName: 'taskId',
          type: {
            name: 'String'
          }
        },
        subtaskId: {
          required: false,
          serializedName: 'subtaskId',
          type: {
            name: 'Number'
          }
        },
        taskState: {
          required: true,
          serializedName: 'taskState',
          type: {
            name: 'Enum',
            allowedValues: [ 'active', 'preparing', 'running', 'completed' ]
          }
        },
        executionInfo: {
          required: false,
          serializedName: 'executionInfo',
          type: {
            name: 'Composite',
            className: 'TaskExecutionInformation'
          }
        }
      }
    }
  };
};

module.exports = TaskInformation;
